@@ -1,5 +1,25 @@
 /*
- * $Id: compalloc.c,v 1.6 2005/07/03 07:01:17 daniels Exp $
+ * $Id: compalloc.c,v 1.7 2005/07/03 07:37:34 daniels Exp $
+ *
+ * Copyright © 2006 Sun Microsystems
+ *
+ * Permission to use, copy, modify, distribute, and sell this software and its
+ * documentation for any purpose is hereby granted without fee, provided that
+ * the above copyright notice appear in all copies and that both that
+ * copyright notice and this permission notice appear in supporting
+ * documentation, and that the name of Sun Microsystems not be used in
+ * advertising or publicity pertaining to distribution of the software without
+ * specific, written prior permission.  Sun Microsystems makes no
+ * representations about the suitability of this software for any purpose.  It
+ * is provided "as is" without express or implied warranty.
+ *
+ * SUN MICROSYSTEMS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
+ * EVENT SHALL SUN MICROSYSTEMS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
+ * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
+ * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+ * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  *
  * Copyright © 2003 Keith Packard
  *
@@ -58,6 +78,11 @@ compRedirectWindow (ClientPtr pClient, WindowPtr pWin, int update)
     CompWindowPtr	cw = GetCompWindow (pWin);
     CompClientWindowPtr	ccw;
     Bool		wasMapped = pWin->mapped;
+    CompScreenPtr       cs = GetCompScreen(pWin->drawable.pScreen);
+    
+    if (pWin == cs->pOverlayWin) {
+	return Success;
+    }
 
     /*
      * Only one Manual update is allowed
@@ -101,7 +126,11 @@ compRedirectWindow (ClientPtr pClient, WindowPtr pWin, int update)
 	    return BadAlloc;
 	}
 	if (wasMapped)
+	{
+	    DisableMapUnmapEvents (pWin);
 	    UnmapWindow (pWin, FALSE);
+	    EnableMapUnmapEvents (pWin);
+	}
 
 	REGION_NULL (pScreen, &cw->borderClip);
 	cw->update = CompositeRedirectAutomatic;
@@ -135,7 +164,9 @@ compRedirectWindow (ClientPtr pClient, WindowPtr pWin, int update)
     {
 	Bool	overrideRedirect = pWin->overrideRedirect;
 	pWin->overrideRedirect = TRUE;
+	DisableMapUnmapEvents (pWin);
 	MapWindow (pWin, pClient);
+	EnableMapUnmapEvents (pWin);
 	pWin->overrideRedirect = overrideRedirect;
     }
     
@@ -169,7 +200,11 @@ compFreeClientWindow (WindowPtr pWin, XID id)
     if (!cw->clients)
     {
 	if (wasMapped)
+	{
+	    DisableMapUnmapEvents (pWin);
 	    UnmapWindow (pWin, FALSE);
+	    EnableMapUnmapEvents (pWin);
+	}
     
 	if (pWin->redirectDraw)
 	    compFreePixmap (pWin);
@@ -193,7 +228,9 @@ compFreeClientWindow (WindowPtr pWin, XID id)
     {
 	Bool	overrideRedirect = pWin->overrideRedirect;
 	pWin->overrideRedirect = TRUE;
+	DisableMapUnmapEvents (pWin);
 	MapWindow (pWin, clients[CLIENT_ID(id)]);
+	EnableMapUnmapEvents (pWin);
 	pWin->overrideRedirect = overrideRedirect;
     }
 }
