@@ -1,4 +1,4 @@
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/common/xf86Config.c,v 1.8 2005/05/18 10:31:53 eich Exp $ */
+/* $XdotOrg: xserver/xorg/hw/xfree86/common/xf86Config.c,v 1.23 2006/03/12 00:11:33 krh Exp $ */
 /* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Config.c,v 3.276 2003/10/08 14:58:26 dawes Exp $ */
 
 
@@ -448,6 +448,52 @@ xf86InputDriverlistFromConfig()
     return modulearray;
 }
 
+static void
+fixup_video_driver_list(char **drivers)
+{
+    static const char *fallback[5] = { "vga", "vesa", "fbdev", "wsfb", NULL };
+    char **end, **drv;
+    char *x;
+    char **ati, **atimisc;
+    int i;
+
+    /* walk to the end of the list */
+    for (end = drivers; *end && **end; end++) ;
+    end--;
+
+    /*
+     * for each of the fallback drivers, if we find it in the list,
+     * swap it with the last available non-fallback driver.
+     */
+    for (i = 0; fallback[i]; i++) {
+        for (drv = drivers; drv != end; drv++) {
+            if (strstr(*drv, fallback[i])) {
+                x = *drv; *drv = *end; *end = x;
+                end--;
+                break;
+            }
+        }
+    }
+    /*
+     * since the ati wrapper driver is gross and awful, sort ati before
+     * atimisc, which makes sure all the ati symbols are visible in xorgcfg.
+     */
+    for (drv = drivers; drv != end; drv++) {
+        if (!strcmp(*drv, "atimisc")) {
+            atimisc = drv;
+            for (drv = atimisc; drv != end; drv++) {
+                if (!strcmp(*drv, "ati")) {
+                    ati = drv;
+                    x = *ati; *ati = *atimisc; *atimisc = x;
+                    return;
+                }
+            }
+            /* if we get here, ati was already ahead of atimisc */
+            return;
+        }
+    }
+}
+
 
 /*
  * Generate a compiled-in list of driver names.  This is used to produce a
@@ -457,6 +503,18 @@ xf86InputDriverlistFromConfig()
 static char **
 GenerateDriverlist(char * dirname, char * drivernames)
 {
+#ifdef XFree86LOADER
+    char **ret;
+    const char *subdirs[] = { dirname, NULL };
+    static const char *patlist[] = {"(.*)_drv\\.so", "(.*)_drv\\.o", NULL};
+    ret = LoaderListDirs(subdirs, patlist);
+    
+    /* fix up the probe order for video drivers */
+    if (strstr(dirname, "drivers"))
+        fixup_video_driver_list(ret);
+
+    return ret;
+#else /* non-loadable server */
     char *cp, **driverlist;
     int count;
 
@@ -487,63 +545,8 @@ GenerateDriverlist(char * dirname, char * drivernames)
     }
     driverlist[count] = NULL;
 
-#ifdef XFree86LOADER
-    {
-        const char *subdirs[] = {NULL, NULL};
-        static const char *patlist[] = {"(.*)_drv\\.so", "(.*)_drv\\.o", NULL};
-        char **dlist, **clist, **dcp, **ccp;
-	int size;
-
-        subdirs[0] = dirname;
-
-        /* Get module list */
-        dlist = LoaderListDirs(subdirs, patlist);
-        if (!dlist) {
-            xfree(driverlist);
-            return NULL;        /* No modules, no list */
-        }
-
-        clist = driverlist;
-
-        /* The resulting list cannot be longer than the module list */
-        for (dcp = dlist, count = 0;  *dcp++;  count++);
-        driverlist = (char **)xnfalloc((size = count + 1) * sizeof(char *));
-
-        /* First, add modules not in compiled-in list */
-        for (count = 0, dcp = dlist;  *dcp;  dcp++) {
-            for (ccp = clist;  ;  ccp++) {
-                if (!*ccp) {
-                    driverlist[count++] = *dcp;
-		    if (count >= size)
-			driverlist = (char**)
-			    xnfrealloc(driverlist, ++size * sizeof(char*));
-                    break;
-                }
-                if (!strcmp(*ccp, *dcp))
-                    break;
-            }
-        }
-
-        /* Next, add compiled-in names that are also modules */
-        for (ccp = clist;  *ccp;  ccp++) {
-            for (dcp = dlist;  *dcp;  dcp++) {
-                if (!strcmp(*ccp, *dcp)) {
-                    driverlist[count++] = *ccp;
-		    if (count >= size)
-			driverlist = (char**)
-			    xnfrealloc(driverlist, ++size * sizeof(char*));
-                    break;
-                }
-            }
-        }
-
-        driverlist[count] = NULL;
-        xfree(clist);
-        xfree(dlist);
-    }
-#endif /* XFree86LOADER */
-
     return driverlist;
+#endif
 }
 
 
@@ -753,7 +756,8 @@ typedef enum {
     FLAG_LOG,
     FLAG_RENDER_COLORMAP_MODE,
     FLAG_HANDLE_SPECIAL_KEYS,
-    FLAG_RANDR
+    FLAG_RANDR,
+    FLAG_AIGLX
 } FlagValues;
    
 static OptionInfoRec FlagOptions[] = {
@@ -823,6 +827,8 @@ static OptionInfoRec FlagOptions[] = {
         {0}, FALSE },
   { FLAG_RANDR,			"RandR",			OPTV_BOOLEAN,
 	{0}, FALSE },
+  { FLAG_AIGLX,			"AIGLX",			OPTV_BOOLEAN,
+	{0}, FALSE },
   { -1,				NULL,				OPTV_NONE,
 	{0}, FALSE },
 };
@@ -1007,6 +1013,13 @@ configServerFlags(XF86ConfFlagsPtr flagsconf, XF86OptionPtr layoutopts)
     else
 	xf86Info.estimateSizesAggressively = 0;
 
+    xf86Info.aiglx = TRUE;
+    xf86Info.aiglxFrom = X_DEFAULT;
+    if (xf86GetOptValBool(FlagOptions, FLAG_AIGLX, &value)) {
+	xf86Info.aiglx = value;
+	xf86Info.aiglxFrom = X_CONFIG;
+    }
+
 /* Make sure that timers don't overflow CARD32's after multiplying */
 #define MAX_TIME_IN_MIN (0x7fffffff / MILLI_PER_MIN)
 
@@ -1106,6 +1119,11 @@ configInputKbd(IDevPtr inputp)
   MessageType from = X_DEFAULT;
   Bool customKeycodesDefault = FALSE;
   int verb = 0;
+#if defined(XQUEUE)
+  char *kbdproto = "Xqueue";
+#else
+  char *kbdproto = "standard";
+#endif
 
   /* Initialize defaults */
   xf86Info.xleds         = 0L;
@@ -1146,16 +1164,25 @@ configInputKbd(IDevPtr inputp)
   xf86Info.xkbgeometry   = NULL;
 #endif
 
-  s = xf86SetStrOption(inputp->commonOptions, "Protocol", "standard");
+  s = xf86SetStrOption(inputp->commonOptions, "Protocol", kbdproto);
   if (xf86NameCmp(s, "standard") == 0) {
      xf86Info.kbdProc    = xf86KbdProc;
      xf86Info.kbdEvents  = xf86KbdEvents;
      xfree(s);
   } else if (xf86NameCmp(s, "xqueue") == 0) {
+#ifdef __UNIXWARE__
+    /*
+     * To retain compatibility with older config files, on UnixWare, we
+     * accept the xqueue protocol but use the normal keyboard procs.
+     */
+     xf86Info.kbdProc    = xf86KbdProc;
+     xf86Info.kbdEvents  = xf86KbdEvents;
+#else
 #ifdef XQUEUE
     xf86Info.kbdProc = xf86XqueKbdProc;
     xf86Info.kbdEvents = xf86XqueEvents;
     xf86Msg(X_CONFIG, "Xqueue selected for keyboard input\n");
+#endif
 #endif
     xfree(s);
 #ifdef WSCONS_SUPPORT
@@ -2122,6 +2149,17 @@ configScreen(confScreenPtr screenp, XF86ConfScreenPtr conf_screen, int scrnum,
     return TRUE;
 }
 
+typedef enum {
+    MON_REDUCEDBLANKING
+} MonitorValues;
+
+static OptionInfoRec MonitorOptions[] = {
+  { MON_REDUCEDBLANKING,      "ReducedBlanking",        OPTV_BOOLEAN,
+       {0}, FALSE },
+  { -1,                                NULL,                   OPTV_NONE,
+       {0}, FALSE },
+};
+
 static Bool
 configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
 {
@@ -2143,6 +2181,7 @@ configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
     monitorp->gamma = zeros;
     monitorp->widthmm = conf_monitor->mon_width;
     monitorp->heightmm = conf_monitor->mon_height;
+    monitorp->reducedblanking = FALSE;
     monitorp->options = conf_monitor->mon_option_lst;
 
     /*
@@ -2262,6 +2301,11 @@ configMonitor(MonPtr monitorp, XF86ConfMonitorPtr conf_monitor)
 	    return FALSE;
     }
 
+    /* Check wether this Monitor accepts Reduced Blanking modelines */
+    xf86ProcessOptions(-1, monitorp->options, MonitorOptions);
+
+    xf86GetOptValBool(MonitorOptions, MON_REDUCEDBLANKING,
+                      &monitorp->reducedblanking);
     return TRUE;
 }
 
@@ -2435,14 +2479,14 @@ configDRI(XF86ConfDRIPtr drip)
 }
 #endif
 
+/* Extension enable/disable in miinitext.c */
+extern Bool EnableDisableExtension(char *name, Bool enable);
+
 static Bool
 configExtensions(XF86ConfExtensionsPtr conf_ext)
 {
     XF86OptionPtr o;
 
-    /* Extension enable/disable in miinitext.c */
-    extern Bool EnableDisableExtension(char *name, Bool enable);
-
     if (conf_ext && conf_ext->ext_option_lst) {
 	for (o = conf_ext->ext_option_lst; o; o = xf86NextOption(o)) {
 	    char *name   = xf86OptionName(o);
@@ -2656,7 +2700,7 @@ xf86HandleConfigFile(Bool autoconfig)
            scanptr = xf86ConfigLayout.screens->screen->device->busID;
     }
     if (scanptr) {
-       int bus, device, func, stroffset = 0;
+       int bus, device, func;
        if (strncmp(scanptr, "PCI:", 4) != 0) {
            xf86Msg(X_WARNING, "Bus types other than PCI not yet isolable.\n"
                               "\tIgnoring IsolateDevice option.\n");
